require 'pnote_client/documents/hml/char'
require 'pnote_client/documents/hml/equation'
require 'pnote_client/documents/hml/table'
require 'pnote_client/documents/hml/rectangle'
require 'pnote_client/documents/hml/image'

module PnoteClient
  module Documents
    class Hml
      class Paragraph
        # Textable Element

        def self.from_tag(pg_tag)
          paragraph = self.new(
            style_id: pg_tag['Style'],
            shape_id: pg_tag['ParaShape']
          )

          # Add Elements
          children = pg_tag.xpath("TEXT/CHAR|TEXT/EQUATION|TEXT/ENDNOTE|TEXT/FOOTNOTE|TEXT/TABLE|TEXT/RECTANGLE|TEXT/PICTURE/IMAGE")
          children.each do |child|
            if child.name == 'CHAR'
              paragraph.add_element(Char.from_tag(child))
            elsif child.name == 'EQUATION'
              paragraph.add_element(Equation.from_tag(child))
            elsif child.name == 'ENDNOTE'
              pg_tags = child.xpath("PARALIST/P")
              pg_tags.map {|pg_tag| self.from_tag(pg_tag)}.each do |endnote_paragraph|
                paragraph.add_endnote_paragraph(endnote_paragraph)
              end
            elsif child.name == 'FOOTNOTE'
              pg_tags = child.xpath("PARALIST/P")
              pg_tags.map {|pg_tag| self.from_tag(pg_tag)}.each do |footnote_paragraph|
                paragraph.add_footnote_paragraph(footnote_paragraph)
              end
            elsif child.name == 'TABLE'
              table = Table.from_tag(child)
              paragraph.add_element(table)
            elsif child.name == 'RECTANGLE'
              rectangle = Rectangle.from_tag(child)
              paragraph.add_element(rectangle)
            elsif child.name = 'IMAGE'
              image = Image.from_tag(child)
              paragraph.add_element(image)
            end
          end

          return paragraph
        end

        attr_reader :style_id, :shape_id, :elements
        attr_reader :endnote_paragraphs, :footnote_paragraphs

        def initialize(style_id:, shape_id:)
          @style_id = style_id
          @shape_id = shape_id
          @endnote_paragraphs = []
          @footnote_paragraphs = []
          @elements = []
        end

        def add_element(element)
          @elements << element
        end

        def add_endnote_paragraph(new_paragraph)
          @endnote_paragraphs << new_paragraph
        end

        def add_footnote_paragraph(new_paragraph)
          @footnote_paragraphs << new_paragraph
        end

        def content
          return @elements.select{|element| element.textable?}.map {|element| element.content}.join
        end

        def textable?
          return true
        end
      end
    end
  end
end
