require 'pnote_client/documents/hml/table_row'

module PnoteClient
  module Documents
    class Hml
      class Table

        def self.from_tag(table_tag)
          row_count = table_tag['RowCount']
          col_count = table_tag['ColCount']

          table = self.new(row_count: row_count.to_i, col_count: col_count.to_i)
          row_tags = table_tag.xpath("ROW")
          row_tags.each do |row_tag|
            table.add_row(TableRow.from_tag(row_tag))
          end

          return table
        end

        attr_reader :row_count, :col_count, :rows

        def initialize(row_count: 0, col_count: 0)
          @row_count = row_count
          @col_count = col_count
          @rows = []
        end

        def add_row(row)
          @rows << row
        end

        def content
          json = {
            col_count: @col_count,
            row_count: @row_count,
            rows: rows_content
          }.to_json

          return "\n![table](#{json})\n"
        end

        def textable?
          return true
        end

        private

        def rows_content
          @rows.map {|row| row.content}
        end
      end
    end
  end
end
