require 'pnote_client/documents/hml/paragraph'

module PnoteClient
  module Documents
    class Hml
      class TableCell
        def self.from_tag(cell_tag)
          row_span = cell_tag['RowSpan']
          col_span = cell_tag['ColSpan']

          cell = self.new(row_span: row_span.to_i, col_span: col_span.to_i)

          pg_tags = cell_tag.xpath("PARALIST/P")
          pg_tags.each do |pg_tag|
            cell.add_paragraph(Paragraph.from_tag(pg_tag))
          end

          return cell
        end

        attr_reader :row_span, :col_span, :paragraphs

        def initialize(row_span: 1, col_span: 1)
          @row_span = row_span
          @col_span = col_span
          @paragraphs = []
        end

        def add_paragraph(new_paragraph)
          @paragraphs << new_paragraph
        end

        def content
          return {
            row_span: @row_span,
            col_span: @col_span,
            content: paragraphs_content
          }
        end

        private

        def paragraphs_content
          return @paragraphs.map {|pg| pg.content}.join("\n")
        end
      end
    end
  end
end
